using FileStorage.Repository.SqlSugar;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.StaticFiles;
using Microsoft.Extensions.Options;
using SqlSugar;
using XF.Domain.Authentications;

namespace FileStorage.Host.Controllers
{
    [ApiController]
    [Route("file")]
    public class FileController : ControllerBase
    {
        private readonly ILogger<FileController> _logger;
        private readonly IFileStorage _fileStorage;
        private readonly ISugarUnitOfWork<FileStorageDbContext> _uow;

        public FileController(
            ILogger<FileController> logger,
            IFileStorage fileStorage,
            ISugarUnitOfWork<FileStorageDbContext> uow
            )
        {
            _logger = logger;
            _fileStorage = fileStorage;
            _uow = uow;
        }
        /// <summary>
        /// ڲãļʽ
        /// </summary>
        /// <param name="fileData"></param>
        /// <param name="source"></param>
        /// <returns></returns>
        [HttpPost("uploadinside")]
        public async Task<IActionResult> UploadInside(IFormFile fileData, string source)
        { 
            var file = await _fileStorage.UploadAsync(fileData.FileName, fileData.Length, "", fileData.OpenReadStream(), source ?? string.Empty, false);
            var Path = await _fileStorage.GetFileUrlIndefinitelyAsync(file.Id, source ?? string.Empty);
            return Ok(new { Id = file.Id, fileName = file.FileName, Path = Path });
        }

        /// <summary>
        /// ļʽ
        /// </summary>
        /// <param name="fileData"></param>
        /// <param name="source"></param>
        /// <returns></returns>
        [HttpPost("upload")]
        public async Task<IActionResult> Upload(IFormFile fileData, string source)
        {
            var file = await _fileStorage.UploadAsync(fileData.FileName, fileData.Length, "", fileData.OpenReadStream(), source ?? string.Empty, true);
            var Path = await _fileStorage.GetFileUrlIndefinitelyAsync(file.Id, source ?? string.Empty);
            return Ok(new { Id = file.Id, fileName = file.FileName, Path = Path });
        }

        [HttpGet("getfileurl")]
        public async Task<IActionResult> GetFileUrl([FromQuery] UploadGetDto dto)
        {
            var uri = await _fileStorage.GetFileUrlAsync(dto.Id, dto.Source ?? string.Empty);
            return Ok(new { Uri = uri });
        }


        [HttpGet("downloadfile")]
        public async Task<IActionResult> DownLoadFile([FromQuery] UploadGetDto dto)
        {
            var (stream, content, fileName) = _fileStorage.DownLoadFile(dto.Id);
            if (stream != null)
            {
                return File(stream, content, fileName);
            }
            return null;
        }

        [HttpGet("delfile")]
        public async Task<bool> DelFile([FromQuery] UploadGetDto dto)
        {
            return await _fileStorage.DelFileAsync(dto.Id, dto.Source ?? string.Empty);
        }

        [HttpGet("file_path")]
        public async Task<IActionResult> GetFilePath([FromQuery] UploadGetDto dto)
        {
            var uri = await _fileStorage.GetFilePathAsync(dto.Id, dto.Source ?? string.Empty);
            return Ok(new { Path = uri });
        }

        [HttpGet("createdb")]
        public Task CreateDb()
        {
            var db = _uow.Db;
            db.DbMaintenance.CreateDatabase();
            db.CodeFirst.InitTables<FileMetadata>();
            return Task.CompletedTask;
        }

        /// <summary>
        /// ļ
        /// </summary>
        /// <param name="id"></param>
        /// <param name="expires"></param>
        /// <param name="clientid"></param>
        /// <param name="signature"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpGet("files")]
        public async Task Files(string id, string expires, string clientid, string signature)
        {
            var fullPath = await _fileStorage.GetFilePath(id, expires, clientid, signature);
            await HttpContext.Response.SendFileAsync(fullPath);
        }

        /// <summary>
        /// ļ
        /// </summary>
        /// <param name="id"></param>
        /// <param name="expires"></param>
        /// <param name="clientid"></param>
        /// <param name="signature"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpGet("files_indefinitely")]
        public async Task FilesIndefinitely(string id, string expires, string clientid, string signature)
        {
            var fullPath = await _fileStorage.GetFilePathIndefinitely(id, expires, clientid, signature);
            await HttpContext.Response.SendFileAsync(fullPath);
        }

        [HttpGet("test")]
        [AllowAnonymous]
        public string Test()
        {
            return DateTime.Now.ToString();
        }
    }
}