﻿using FileStorage.Repository.SqlSugar.Extensions;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Server.Kestrel.Core;
using Microsoft.Extensions.FileProviders;
using Serilog;
using XF.Domain.Dependency;
using XF.Domain.Filters;

namespace FileStorage.Host;

internal static class StartupExtensions
{
    const string CorsOrigins = "CorsOrigins";
    internal static WebApplication ConfigureServices(this WebApplicationBuilder builder)
    {
        var services = builder.Services;
        var configuration = builder.Configuration;

        services.AddHttpContextAccessor();

#if DEBUG
        builder.WebHost.UseUrls("http://*:50102");
#endif

        services.Configure<StorageConfiguration>(d => configuration.GetSection(nameof(StorageConfiguration)).Bind(d));
        // Add services to the container.
        services.BatchInjectServices();

        services.AddScoped<IFileStorage, DefaultFileStorage>();


        //var storageConfig = configuration.GetSection(nameof(StorageConfiguration)).Get<StorageConfiguration>();
        //if(storageConfig == null || string.IsNullOrEmpty(storageConfig.Impt) || storageConfig.Impt == FileStorageConsts.LocalImpt)
        //{

        //}

        services.AddControllers(options =>
        {
            options.Filters.Add<UnifyResponseFilter>();
            options.Filters.Add<UserFriendlyExceptionFilter>();
        });

        // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
        services.AddEndpointsApiExplorer();

        //取消请求体大小限制
        services.Configure<KestrelServerOptions>(d =>
        {
            d.Limits.MaxRequestBodySize = null;
        });

        services
            //swagger
            .RegisterSwagger()
            // CORS
            .RegisterCors(configuration, CorsOrigins)
            //mapster
            .RegisterMapper()
            //sqlsugar
            .AddSqlSugar(configuration);

        ////auth
        //var ids4Options = configuration.GetSection(nameof(Ids4AuthorizeOptions)).Get<Ids4AuthorizeOptions>();
        //services.RegisterIds4Authorize(ids4Options);

        ////mediatr
        //services.AddMediatR(d =>
        //{
        //    d.RegisterServicesFromAssembly(typeof(ApplicationStartupExtensions).Assembly);
        //});


        ////cache
        //services.AddCache(d =>
        //    {
        //        d.ConnectionString = configuration.GetConnectionString("Redis") ?? string.Empty;
        //        d.Prefix = "Hotline";
        //        d.TopicName = "hotline-topic";
        //    });

        ////validator
        //services.AddFluentValidationAutoValidation(config =>
        //{
        //    config.DisableDataAnnotationsValidation = true;
        //})
        //    .AddValidatorsFromAssembly(typeof(AppContractsStartupExtensions).Assembly);

        return builder.Build();
    }

    internal static WebApplication ConfigurePipelines(this WebApplication app)
    {
        app.UseSerilogRequestLogging();

        var swaggerEnable = app.Configuration.GetSection("Swagger").Get<bool>();
        // Configure the HTTP request pipeline.
        if (swaggerEnable)
        {
            app.UseSwagger();
            app.UseSwaggerUI();
        }
        app.UseCors(CorsOrigins);

        //app.UseAuthentication();
        app.UseAuthorization();

        if (!Directory.Exists(Path.Combine(Directory.GetCurrentDirectory(), "uploads")))
        {
            Directory.CreateDirectory(Path.Combine(Directory.GetCurrentDirectory(), "uploads"));
        };
        app.UseStaticFiles(new StaticFileOptions
        {
            FileProvider = new PhysicalFileProvider(Path.Combine(Directory.GetCurrentDirectory(), "uploads")),
            RequestPath = new PathString("/uploads")
        });

        app.MapControllers();

        return app;
    }
}